// ***************************************************************************
// LICENSING:  This is free, open source software, licensed under
//             Version 2 of the GNU General Public License (GPL).
//
// In particular, this means that distribution of this software in a binary
// format, e.g. as compiled in as part of a .ex4 format, must be accompanied
// by the non-obfuscated source code of both this file, AND the .mq4 source
// files which it is compiled with, or you must make such files available at
// no charge to binary recipients. If you do not agree with such terms you
// must not use this code.  Detailed terms of the GPL are widely available
// on the Internet.  The Library GPL (LGPL) was intentionally not used,
// therefore the source code of files which link to this are subject to
// terms of the GPL if binaries made from them are publicly distributed or
// sold.
//
// ANY USE OF THIS CODE NOT CONFORMING TO THIS LICENSE MUST FIRST RECEIVE
// PRIOR AUTHORIZATION FROM THE AUTHOR(S).  ANY COMMERCIAL USE MUST FIRST
// OBTAIN A COMMERCIAL LICENSE FROM THE AUTHOR(S).
//
// Copyright (2013), Andrew Sumner, andrewsumner@yahoo.com
// ***************************************************************************

// ***************************************************************************
// HISTORY:
//
// v1 20130429, Andrew:-
//   - Functionality equivalent with version 04c of the SS_SupportResistance 
//     indicator, minus the drawing of boxes, setting of global variables 
//     and generation of alert messages.
//
// v2 20130507, Andrew:-
//   - Improved handling of SSSR_FindZone() when the price is within a zone.
//   - Added SSSR_FindZoneV2() to the header file.
//
// ***************************************************************************

#property copyright "Copyright  2013, Andrew Sumner"
#property link "andrewsumner@yahoo.com"
#property library

int     SSSR_BackLimit   = 10000;

double  SSSR_zone.fuzzfactor = 0.9;
bool    SSSR_zone.merge = true;
bool    SSSR_zone.extend = true;

double  SSSR_zone.fastfactor = 3.0;
double  SSSR_zone.slowfactor = 6.0;

double  SSSR_FastDnPts[], SSSR_FastUpPts[];
double  SSSR_SlowDnPts[], SSSR_SlowUpPts[];

double  SSSR_zone.hi[1000], SSSR_zone.lo[1000];
int     SSSR_zone.start[1000], SSSR_zone.hits[1000], SSSR_zone.type[1000], SSSR_zone.strength[1000], SSSR_zone.count = 0;
bool    SSSR_zone.turn[1000];
string  SSSR_sym;
int     SSSR_TF;

#define SSSR_ZONE_SUPPORT   1
#define SSSR_ZONE_RESIST    2

#define SSSR_ZONE_WEAK      0
#define SSSR_ZONE_TURNCOAT  1
#define SSSR_ZONE_UNTESTED  2
#define SSSR_ZONE_VERIFIED  3
#define SSSR_ZONE_PROVEN    4

#define SSSR_UP_POINT       1
#define SSSR_DN_POINT      -1

#define SSSR_UP 1
#define SSSR_DN -1

///////////////////////////////////////////////////////////////////////////////////////////////
// INTERNAL LIBRARY FUNCTIONS ONLY BELOW
///////////////////////////////////////////////////////////////////////////////////////////////

//
// SSSR_Settings
//
// Call this once when the EA inits
//
void SSSR_Settings(int backlimit, double fuzzfactor, bool merge, bool extend, double fastfactor, double slowfactor)
{
   if (backlimit > 0)
      SSSR_BackLimit = backlimit;

   if (fuzzfactor > 0)
      SSSR_zone.fuzzfactor = fuzzfactor;

   SSSR_zone.merge = merge;
   SSSR_zone.extend = extend;

   if (fastfactor > 0)
      SSSR_zone.fastfactor = fastfactor;

   if (slowfactor > 0)
      SSSR_zone.slowfactor = slowfactor;
}

//
// SSSR_UpdateZones
//
// Call this to make the library recalculate the zones
//
void SSSR_UpdateZones(bool checkCandle, string symbol, int timeframe)
{
   if (SSSR_sym != symbol || SSSR_TF != timeframe)
      checkCandle = false;

   SSSR_sym = symbol;
   SSSR_TF = timeframe;

   if (checkCandle && !SSSR_NewBar())
      return;

   SSSR_FastFractals();
   SSSR_SlowFractals();
   SSSR_FindZones();

   Print(SSSR_zone.count+" zones identified");
}

//
// SSSR_FindZone
//
// Call this function to discover the closest zone to the provided price value
//
int SSSR_FindZone(int direction, bool useWeak, double price)
{
   int i, zone = -1;
   double hi = 0, lo = 99999;

   if (direction == SSSR_UP)
   {
      for (i = 0; i < SSSR_zone.count; i++)
      {
         if (SSSR_zone.hi[i] > price && (useWeak || SSSR_zone.strength[i] != SSSR_ZONE_WEAK))
         {
            if (SSSR_zone.lo[i] < lo && (SSSR_zone.type[i] == SSSR_ZONE_RESIST || SSSR_zone.lo[i] > price))
            {
               lo = SSSR_zone.lo[i];
               zone = i;
            }
         }
      }
   }
   else if (direction == SSSR_DN)
   {
      for (i = 0; i < SSSR_zone.count; i++)
      {
         if (SSSR_zone.lo[i] < price && (useWeak || SSSR_zone.strength[i] != SSSR_ZONE_WEAK))
         {
            if (SSSR_zone.hi[i] > hi && (SSSR_zone.type[i] == SSSR_ZONE_SUPPORT || SSSR_zone.hi[i] < price))
            {
               hi = SSSR_zone.hi[i];
               zone = i;
            }
         }
      }
   }
   else
      return(-1);

   return(zone);
}

int SSSR_GetZoneType(int zone)
{
   return(SSSR_zone.type[zone]);
}

double SSSR_GetZoneHi(int zone)
{
   if (zone < 0 || zone >= SSSR_zone.count)
      return(0);

   return(SSSR_zone.hi[zone]);
}

double SSSR_GetZoneLo(int zone)
{
   if (zone < 0 || zone >= SSSR_zone.count)
      return(0);

   return(SSSR_zone.lo[zone]);
}

int SSSR_GetZoneStrength(int zone)
{
   if (zone < 0 || zone >= SSSR_zone.count)
      return(0);

   return(SSSR_zone.strength[zone]);
}

///////////////////////////////////////////////////////////////////////////////////////////////
// INTERNAL LIBRARY FUNCTIONS ONLY BELOW
///////////////////////////////////////////////////////////////////////////////////////////////

bool SSSR_NewBar()
{
   static datetime LastTime = 0;

   if (iTime(SSSR_sym, SSSR_TF, 0) > LastTime)
   {
      LastTime = iTime(SSSR_sym, SSSR_TF, 0);
      return(true);
   }

   else
      return(false);
}

bool SSSR_Fractal(int M, int P, int shift)
{
   if (SSSR_TF > P)
      P = SSSR_TF;

   P = P / SSSR_TF * 2 + MathCeil(P / SSSR_TF / 2);

   if (shift < P)
      return(false);

   if (shift > iBars(SSSR_sym, SSSR_TF) - P)
      return(false);

   for (int i = 1; i <= P; i++)
   {
      if (M == SSSR_UP_POINT)
      {
         if (iHigh(SSSR_sym, SSSR_TF, shift + i) > iHigh(SSSR_sym, SSSR_TF, shift))
            return(false);

         if (iHigh(SSSR_sym, SSSR_TF, shift - i) >= iHigh(SSSR_sym, SSSR_TF, shift))
            return(false);
      }

      if (M == SSSR_DN_POINT)
      {
         if (iLow(SSSR_sym, SSSR_TF, shift + i) < iLow(SSSR_sym, SSSR_TF, shift))
            return(false);

         if (iLow(SSSR_sym, SSSR_TF, shift - i) <= iLow(SSSR_sym, SSSR_TF, shift))
            return(false);
      }
   }

   return(true);
}

void SSSR_FastFractals()
{
   int shift;
   int limit = MathMin(iBars(SSSR_sym, SSSR_TF) - 1, SSSR_BackLimit);
   int P = SSSR_TF * SSSR_zone.fastfactor;

   ArrayResize(SSSR_FastUpPts, limit);
   ArrayResize(SSSR_FastDnPts, limit);
   ArraySetAsSeries(SSSR_FastUpPts, true);
   ArraySetAsSeries(SSSR_FastDnPts, true);

   SSSR_FastUpPts[0] = 0.0;
   SSSR_FastUpPts[1] = 0.0;
   SSSR_FastDnPts[0] = 0.0;
   SSSR_FastDnPts[1] = 0.0;

   for (shift = limit; shift > 1; shift--)
   {
      SSSR_FastUpPts[shift] = 0.0;

      if (SSSR_Fractal(SSSR_UP_POINT, P, shift))
         SSSR_FastUpPts[shift] = iHigh(SSSR_sym, SSSR_TF, shift);

      SSSR_FastDnPts[shift] = 0.0;

      if (SSSR_Fractal(SSSR_DN_POINT, P, shift))
         SSSR_FastDnPts[shift] = iLow(SSSR_sym, SSSR_TF, shift);
   }
}

void SSSR_SlowFractals()
{
   int shift;
   int limit = MathMin(iBars(SSSR_sym, SSSR_TF) - 1, SSSR_BackLimit);
   int P = Period() * SSSR_zone.slowfactor;

   ArrayResize(SSSR_SlowUpPts, limit);
   ArrayResize(SSSR_SlowDnPts, limit);
   ArraySetAsSeries(SSSR_SlowUpPts, true);
   ArraySetAsSeries(SSSR_SlowDnPts, true);

   SSSR_SlowUpPts[0] = 0.0;
   SSSR_SlowUpPts[1] = 0.0;
   SSSR_SlowDnPts[0] = 0.0;
   SSSR_SlowDnPts[1] = 0.0;

   for (shift = limit; shift > 1; shift--)
   {
      SSSR_SlowUpPts[shift] = 0.0;

      if (SSSR_Fractal(SSSR_UP_POINT, P, shift))
         SSSR_SlowUpPts[shift] = iHigh(SSSR_sym, SSSR_TF, shift);

      SSSR_SlowDnPts[shift] = 0.0;

      if (SSSR_Fractal(SSSR_DN_POINT, P, shift))
         SSSR_SlowDnPts[shift] = iLow(SSSR_sym, SSSR_TF, shift);
   }
}

void SSSR_FindZones()
{
   int i, j, shift, bustcount = 0, testcount = 0;
   double hival, loval;
   bool turned = false, hasturned = false;

   double    temp.hi[1000], temp.lo[1000];
   int       temp.start[1000], temp.hits[1000], temp.strength[1000], temp.count = 0;
   bool      temp.turn[1000], temp.merge[1000];
   int    merge1[1000], merge2[1000], merge.count = 0;

   // iterate through zones from oldest to youngest (ignore recent 5 bars),
   // finding those that have survived through to the present...
   for (shift = MathMin(iBars(SSSR_sym, SSSR_TF) - 1, SSSR_BackLimit); shift > 5; shift--)
   {
      double atr = iATR(SSSR_sym, SSSR_TF, 7, shift);
      double fu = atr / 2 * SSSR_zone.fuzzfactor;
      bool isWeak;
      bool touchOk = false;
      bool isBust = false;
      double close = iClose(SSSR_sym, SSSR_TF, shift);
      double high  = iHigh(SSSR_sym, SSSR_TF, shift);
      double low   = iLow(SSSR_sym, SSSR_TF, shift);
      double hi_i;
      double lo_i;

      if (SSSR_FastUpPts[shift] > 0.001)
      {
         // a fractal high point
         isWeak = true;

         if (SSSR_SlowUpPts[shift] > 0.001)
            isWeak = false;

         hival = high;

         if (SSSR_zone.extend == true)
            hival += fu;

         loval = MathMax(MathMin(close, high - fu), high - fu * 2);
         turned = false;
         hasturned = false;
         isBust = false;

         bustcount = 0;
         testcount = 0;

         for (i = shift - 1; i >= 0; i--)
         {
            hi_i = iHigh(SSSR_sym, SSSR_TF, i);
            lo_i = iLow(SSSR_sym, SSSR_TF, i);

            if ((!turned && SSSR_FastUpPts[i] >= loval && SSSR_FastUpPts[i] <= hival)
                  || (turned && SSSR_FastDnPts[i] <= hival && SSSR_FastDnPts[i] >= loval))
            {
               // Potential touch, just make sure its been 10+candles since the prev one
               touchOk = true;

               for (j = i + 1; j < i + 11; j++)
               {
                  if ((!turned && SSSR_FastUpPts[j] >= loval && SSSR_FastUpPts[j] <= hival)
                        || (turned && SSSR_FastDnPts[j] <= hival && SSSR_FastDnPts[j] >= loval))
                  {
                     touchOk = false;
                     break;
                  }
               }

               if (touchOk)
               {
                  // we have a touch.  If its been busted once, remove bustcount
                  // as we know this level is still valid & has just switched sides
                  bustcount = 0;
                  testcount++;
               }
            }

            if ((turned == false && hi_i > hival)
                  || (turned == true && lo_i < loval))
            {
               // this level has been busted at least once
               bustcount++;

               if (bustcount > 1 || isWeak == true)
               {
                  // busted twice or more
                  isBust = true;
                  break;
               }

               turned = !turned;

               hasturned = true;

               // forget previous hits
               testcount = 0;
            }
         }

         if (!isBust)
         {
            // level is still valid, add to our list
            temp.hi[temp.count] = hival;
            temp.lo[temp.count] = loval;
            temp.turn[temp.count] = hasturned;
            temp.hits[temp.count] = testcount;
            temp.start[temp.count] = shift;
            temp.merge[temp.count] = false;

            if (testcount > 3)
               temp.strength[temp.count] = SSSR_ZONE_PROVEN;

            else
               if (testcount > 0)
                  temp.strength[temp.count] = SSSR_ZONE_VERIFIED;

               else
                  if (hasturned)
                     temp.strength[temp.count] = SSSR_ZONE_TURNCOAT;

                  else
                     if (!isWeak)
                        temp.strength[temp.count] = SSSR_ZONE_UNTESTED;

                     else
                        temp.strength[temp.count] = SSSR_ZONE_WEAK;

            temp.count++;
         }
      }

      else
         if (SSSR_FastDnPts[shift] > 0.001)
         {
            // a zigzag low point
            isWeak = true;

            if (SSSR_SlowDnPts[shift] > 0.001)
               isWeak = false;

            loval = low;

            if (SSSR_zone.extend == true)
               loval -= fu;

            hival = MathMin(MathMax(close, low + fu), low + fu * 2);
            turned = false;
            hasturned = false;

            bustcount = 0;
            testcount = 0;
            isBust = false;

            for (i = shift - 1; i >= 0; i--)
            {
               hi_i = iHigh(SSSR_sym, SSSR_TF, i);
               lo_i = iLow(SSSR_sym, SSSR_TF, i);

               if ((turned && SSSR_FastUpPts[i] >= loval && SSSR_FastUpPts[i] <= hival)
                     || (!turned && SSSR_FastDnPts[i] <= hival && SSSR_FastDnPts[i] >= loval))
               {
                  // Potential touch, just make sure its been 10+candles since the prev one
                  touchOk = true;

                  for (j = i + 1; j < i + 11; j++)
                  {
                     if ((turned && SSSR_FastUpPts[j] >= loval && SSSR_FastUpPts[j] <= hival)
                           || (!turned && SSSR_FastDnPts[j] <= hival && SSSR_FastDnPts[j] >= loval))
                     {
                        touchOk = false;
                        break;
                     }
                  }

                  if (touchOk)
                  {
                     // we have a touch.  If its been busted once, remove bustcount
                     // as we know this level is still valid & has just switched sides
                     bustcount = 0;
                     testcount++;
                  }
               }

               if ((turned && hi_i > hival)
                     || (!turned && lo_i < loval))
               {
                  // this level has been busted at least once
                  bustcount++;

                  if (bustcount > 1 || isWeak)
                  {
                     // busted twice or more
                     isBust = true;
                     break;
                  }

                  turned = !turned;

                  hasturned = true;

                  // forget previous hits
                  testcount = 0;
               }
            }

            if (!isBust)
            {
               // level is still valid, add to our list
               temp.hi[temp.count] = hival;
               temp.lo[temp.count] = loval;
               temp.turn[temp.count] = hasturned;
               temp.hits[temp.count] = testcount;
               temp.start[temp.count] = shift;
               temp.merge[temp.count] = false;

               if (testcount > 3)
                  temp.strength[temp.count] = SSSR_ZONE_PROVEN;

               else
                  if (testcount > 0)
                     temp.strength[temp.count] = SSSR_ZONE_VERIFIED;

                  else
                     if (hasturned)
                        temp.strength[temp.count] = SSSR_ZONE_TURNCOAT;

                     else
                        if (!isWeak)
                           temp.strength[temp.count] = SSSR_ZONE_UNTESTED;

                        else
                           temp.strength[temp.count] = SSSR_ZONE_WEAK;

               temp.count++;
            }
         }
   }

   // look for overlapping zones...
   if (SSSR_zone.merge)
   {
      merge.count = 1;
      int iterations = 0;

      while (merge.count > 0 && iterations < 3)
      {
         merge.count = 0;
         iterations++;

         for (i = 0; i < temp.count; i++)
            temp.merge[i] = false;

         for (i = 0; i < temp.count - 1; i++)
         {
            if (temp.hits[i] == -1 || temp.merge[j])
               continue;

            for (j = i + 1; j < temp.count; j++)
            {
               if (temp.hits[j] == -1 || temp.merge[j])
                  continue;

               if ((temp.hi[i] >= temp.lo[j] && temp.hi[i] <= temp.hi[j])
                     || (temp.lo[i] <= temp.hi[j] && temp.lo[i] >= temp.lo[j])
                     || (temp.hi[j] >= temp.lo[i] && temp.hi[j] <= temp.hi[i])
                     || (temp.lo[j] <= temp.hi[i] && temp.lo[j] >= temp.lo[i]))
               {
                  merge1[merge.count] = i;
                  merge2[merge.count] = j;
                  temp.merge[i] = true;
                  temp.merge[j] = true;
                  merge.count++;
               }
            }
         }

         // ... and merge them ...
         for (i = 0; i < merge.count; i++)
         {
            int target = merge1[i];
            int source = merge2[i];

            temp.hi[target] = MathMax(temp.hi[target], temp.hi[source]);
            temp.lo[target] = MathMin(temp.lo[target], temp.lo[source]);
            temp.hits[target] += temp.hits[source];
            temp.start[target] = MathMax(temp.start[target], temp.start[source]);
            temp.strength[target] = MathMax(temp.strength[target], temp.strength[source]);

            if (temp.hits[target] > 3)
               temp.strength[target] = SSSR_ZONE_PROVEN;

            if (temp.hits[target] == 0 && !temp.turn[target])
            {
               temp.hits[target] = 1;

               if (temp.strength[target] < SSSR_ZONE_VERIFIED)
                  temp.strength[target] = SSSR_ZONE_VERIFIED;
            }

            if (!temp.turn[target] || !temp.turn[source])
               temp.turn[target] = false;

            if (temp.turn[target])
               temp.hits[target] = 0;

            temp.hits[source] = -1;
         }
      }
   }

   // copy the remaining list into our official zones arrays
   SSSR_zone.count = 0;

   for (i = 0; i < temp.count; i++)
   {
      if (temp.hits[i] >= 0 && SSSR_zone.count < 1000)
      {
         SSSR_zone.hi[SSSR_zone.count]       = temp.hi[i];
         SSSR_zone.lo[SSSR_zone.count]       = temp.lo[i];
         SSSR_zone.hits[SSSR_zone.count]     = temp.hits[i];
         SSSR_zone.turn[SSSR_zone.count]     = temp.turn[i];
         SSSR_zone.start[SSSR_zone.count]    = temp.start[i];
         SSSR_zone.strength[SSSR_zone.count] = temp.strength[i];

         if (SSSR_zone.hi[SSSR_zone.count] < Close[4])
            SSSR_zone.type[SSSR_zone.count] = SSSR_ZONE_SUPPORT;

         else
            if (SSSR_zone.lo[SSSR_zone.count] > Close[4])
               SSSR_zone.type[SSSR_zone.count] = SSSR_ZONE_RESIST;

            else
            {
               for (j = 5; j < 1000; j++)
               {
                  if (iClose(SSSR_sym, SSSR_TF, j) < SSSR_zone.lo[SSSR_zone.count])
                  {
                     SSSR_zone.type[SSSR_zone.count] = SSSR_ZONE_RESIST;
                     break;
                  }

                  else
                     if (iClose(SSSR_sym, SSSR_TF, j) > SSSR_zone.hi[SSSR_zone.count])
                     {
                        SSSR_zone.type[SSSR_zone.count] = SSSR_ZONE_SUPPORT;
                        break;
                     }
               }

               if (j == 1000)
                  SSSR_zone.type[SSSR_zone.count] = SSSR_ZONE_SUPPORT;
            }

         SSSR_zone.count++;
      }
   }
}
